@@ -3,7 +3,7 @@ import sys
 import time
 from collections import abc
 from functools import singledispatch
-from inspect import iscoroutinefunction, isawaitable
+from inspect import isawaitable, iscoroutinefunction
 from typing import (
     Any,
     AsyncIterable,
@@ -16,17 +16,16 @@ from typing import (
     Optional,
     Tuple,
     TypeVar,
-    Union,
 )
 
-import zmq
 from IPython import get_ipython
-from tornado.queues import QueueEmpty
 
 from ._async_thread import AsyncThread
 
 T = TypeVar("T")
 
+ZMQ_POLLOUT = 2  # zmq.POLLOUT without zmq dependency
+
 
 class KernelWrapper:
     _current: Optional["KernelWrapper"] = None
@@ -53,7 +52,7 @@ class KernelWrapper:
             # ipykernel < 6
             kernel.shell_handlers["execute_request"] = self._execute_request
 
-        shell.events.register("post_run_cell", self._post_run_cell_hook)
+        shell.events.register("post_execute", self._post_execute_hook)
 
     def restore(self):
         if self._backup_execute_request is not None:
@@ -97,22 +96,25 @@ class KernelWrapper:
                 sys.stderr.flush()
                 if shell_stream is not None:  # 6+
                     kernel._publish_status("idle", "shell")
-                    shell_stream.flush(zmq.POLLOUT)
+                    shell_stream.flush(ZMQ_POLLOUT)
                 else:
                     kernel._publish_status("idle")
 
     async def do_one_iteration(self):
         try:
-            await self._kernel.do_one_iteration()
-            # reset stdio back to original cell
-            self._reset_output()
-        except QueueEmpty:
+            rr = self._kernel.do_one_iteration()
+            if isawaitable(rr):
+                await rr
+        except Exception:  # pylint: disable=broad-except
             # it's probably a bug in ipykernel,
             # .do_one_iteration() should not throw
             return
+        finally:
+            # reset stdio back to original cell
+            self._reset_output()
 
-    def _post_run_cell_hook(self, _):
-        self._shell.events.unregister("post_run_cell", self._post_run_cell_hook)
+    def _post_execute_hook(self, *args, **kw):
+        self._shell.events.unregister("post_execute", self._post_execute_hook)
         self.restore()
         KernelWrapper._current = None
         asyncio.ensure_future(self.replay(), loop=self._loop)
@@ -148,7 +150,9 @@ class KernelWrapper:
 
 class IteratorWrapperAsync(abc.AsyncIterable, Generic[T]):
     def __init__(
-        self, its: AsyncIterable[T], n: int = 1,
+        self,
+        its: AsyncIterable[T],
+        n: int = 1,
     ):
         self._its = its
         self._n = n
@@ -167,7 +171,9 @@ class IteratorWrapperAsync(abc.AsyncIterable, Generic[T]):
 
 class IteratorWrapper(abc.Iterable, Generic[T]):
     def __init__(
-        self, its: Iterable[T], n: int = 1,
+        self,
+        its: Iterable[T],
+        n: int = 1,
     ):
         self._its = its
         self._n = n
@@ -319,7 +325,6 @@ def run_ui_poll_loop(
 
     for x in with_ui_events(as_iterator(f, sleep), n):
         if x is not None:
-            break
+            return x
 
-    assert x is not None
-    return x
+    raise RuntimeError("hm...")  # for mypy sake
